import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:logger/logger.dart';
import 'package:suga_core/src/http/http_error.dart';
import 'package:suga_core/src/http/oauth2_manager.dart';

typedef UnauthorizedResponseHandler = void Function({
  @required HttpError httpError,
});

typedef ErrorMessageParser = String Function({
  @required HttpError httpError,
});

class HttpClientWrapper {
  final Dio dio = Dio();
  final Logger logger;
  final UnauthorizedResponseHandler unauthorizedResponseHandler;
  final Oauth2Manager oauth2Manager;
  final ErrorMessageParser errorMessageParser;

  HttpClientWrapper({
    @required BaseOptions options,
    this.logger,
    bool verbose = false,
    this.unauthorizedResponseHandler,
    this.oauth2Manager,
    List<Interceptor> extraInterceptors,
    this.errorMessageParser = laravelErrorMessageParser,
  }) {
    dio.options = options;
    dio.interceptors.add(InterceptorsWrapper(onRequest: onRequest, onError: onError));

    if (logger != null) {
      if (verbose) {
        dio.interceptors.add(LogInterceptor(logPrint: logger.v, requestBody: true, responseBody: true));
      } else {
        dio.interceptors.add(LogInterceptor(logPrint: logger.d));
      }
    }

    if (extraInterceptors != null) {
      dio.interceptors.addAll(extraInterceptors);
    }
  }

  Future<RequestOptions> onRequest(RequestOptions option) async {
    option.headers["Accept"] = "application/json";

    String accessToken = await oauth2Manager?.getAccessToken();
    if (accessToken != null) {
      option.headers["Authorization"] = "Bearer $accessToken";
    }

    return option;
  }

  HttpError onError(DioError e) {
    final HttpError httpError = HttpError(
      dioError: e,
      errorMessageParser: errorMessageParser,
    );

    if (httpError.response?.statusCode == HttpStatus.unauthorized) {
      logger?.w("Http Unauthorized Response");
      if (unauthorizedResponseHandler != null) {
        unauthorizedResponseHandler(httpError: httpError);
      }
    }

    return httpError;
  }
}
